import { HttpClient } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { ConfigService } from '@services/config.service';
import { MatTabChangeEvent } from '@angular/material/tabs';
import {ToastrService} from "ngx-toastr";

@Component({
  selector: 'app-global-status',
  templateUrl: './global-status.component.html',
  styleUrl: './global-status.component.css'
})
export class GlobalStatusComponent implements OnInit {
  baseUrl: string;
  loading: boolean = false;
  loadingOgBoot: boolean = false;
  loadingDhcp: boolean = false;
  loadingRepositories: boolean = false;
  errorOgBoot: boolean = false;
  errorDhcp: boolean = false;
  errorRepositories: { [key: string]: boolean } = {};
  installedOgLives: any[] = [];
  subnets: any[] = [];
  showLabels: boolean = true;
  isDoughnut: boolean = true;
  colorScheme: any = {
    domain: ['#df200d', '#26a700']
  };
  view: [number, number] = [400, 220];
  repositoriesUrl: string;
  repositories: any[] = [];
  repositoryStatuses: { [key: string]: any } = {};
  lastUpdateTime: string = '';
  selectedRepositoryUuid: string = '';

  ogBootApiUrl: string;
  ogBootDiskUsage: any = {};
  ogBootServicesStatus: any = {};
  ogBootDiskUsageChartData: any[] = [];

  dhcpApiUrl: string;
  dhcpDiskUsage: any = {};
  dhcpServicesStatus: any = {};
  dhcpDiskUsageChartData: any[] = [];
  isDhcp: boolean = false;
  isRepository: boolean = false;

  constructor(
    private configService: ConfigService,
    private toastService: ToastrService,
    private http: HttpClient
  ) {
    this.baseUrl = this.configService.apiUrl;
    this.ogBootApiUrl = `${this.baseUrl}/og-boot/status`;
    this.dhcpApiUrl = `${this.baseUrl}/og-dhcp/status`;
    this.repositoriesUrl = `${this.baseUrl}/image-repositories`;
    
    this.ogBootDiskUsageChartData = [];
    this.dhcpDiskUsageChartData = [];
  }

  ngOnInit(): void {
    this.updateLastUpdateTime();
    
    this.loadOgBootStatus();
    this.loadDhcpStatus();
    this.loadRepositories(false);
    
    this.syncSubnets();
    this.syncTemplates();
    this.syncOgLives();
  }

  syncSubnets() {
    const timeoutId = setTimeout(() => {
      this.toastService.error('Error al sincronizar las subredes: tiempo de espera agotado');
    }, 3500);

    this.http.post(`${this.baseUrl}/subnets/sync`, {}).subscribe({
      next: (response) => {
        clearTimeout(timeoutId);
        this.toastService.success('Sincronización con componente DHCP exitosa');
      },
      error: (error) => {
        clearTimeout(timeoutId);
        this.toastService.error(error.error['hydra:description'] || 'Error al sincronizar las subredes');
      }
    });
  }

  syncTemplates() {
    const timeoutId = setTimeout(() => {
      this.toastService.error('Error al sincronizar las plantillas Pxe: tiempo de espera agotado');
    }, 3500);

    this.http.post(`${this.baseUrl}/pxe-templates/sync`, {})
      .subscribe(response => {
        clearTimeout(timeoutId);
        this.toastService.success('Sincronización de las plantillas Pxe completada');
      }, error => {
        clearTimeout(timeoutId);
        this.toastService.error(error.error['hydra:description'] || 'Error al sincronizar las plantillas Pxe');
      });
  }

  syncOgLives(): void {
    const timeoutId = setTimeout(() => {
      this.toastService.error('Error al sincronizar las imagenes ogLive : tiempo de espera agotado');
    }, 3500);

    this.http.post(`${this.baseUrl}/og-lives/sync`, {})
      .subscribe(response => {
        clearTimeout(timeoutId);
        this.toastService.success('Sincronización con los ogLives completada');
      }, error => {
        clearTimeout(timeoutId);
        this.toastService.error(error.error['hydra:description'] || 'Error al sincronizar las imagenes ogLive');
      });
  }

  [key: string]: any;

  loadStatus(apiUrl: string, diskUsage: any, servicesStatus: any, diskUsageChartData: any[], installedOgLives: any[], isDhcp: boolean, errorState: string, showLoading: boolean = true): void {
    if (isDhcp) {
      this.loadingDhcp = true;
    } else {
      this.loadingOgBoot = true;
    }
    
    if (showLoading) {
      this.loading = true;
    }
    this[errorState] = false;
    const timeoutId = setTimeout(() => {
      if (showLoading) {
        this.loading = false;
      }
      if (isDhcp) {
        this.loadingDhcp = false;
      } else {
        this.loadingOgBoot = false;
      }
      this[errorState] = true;
    }, 3500);
    this.http.get<any>(apiUrl).subscribe({
      next: data => {
        diskUsage.used = data.message.disk_usage.used;
        diskUsage.available = data.message.disk_usage.available;
        diskUsage.total = data.message.disk_usage.total;
        diskUsage.percentage = data.message.disk_usage.percentage;

        Object.assign(servicesStatus, data.message.services_status);

        if (isDhcp) {
          this.subnets.length = 0;
          if (data.message.subnets) {
            this.subnets.push(...data.message.subnets);
          }
        } else {
          installedOgLives.length = 0;
          if (data.message.installed_oglives) {
            installedOgLives.push(...data.message.installed_oglives);
          }
        }

        diskUsageChartData.length = 0;
        diskUsageChartData.push(
          { name: 'Usado', value: parseFloat(diskUsage.used) },
          { name: 'Disponible', value: parseFloat(diskUsage.available) }
        );

        if (showLoading) {
          this.loading = false;
        }
        if (isDhcp) {
          this.loadingDhcp = false;
        } else {
          this.loadingOgBoot = false;
        }
        clearTimeout(timeoutId);
      },
      error: error => {
        this.toastService.error(error.error['hydra:description'] || 'Error al cargar el estado de ogBoot');
        if (showLoading) {
          this.loading = false;
        }
        if (isDhcp) {
          this.loadingDhcp = false;
        } else {
          this.loadingOgBoot = false;
        }
        this[errorState] = true;
        clearTimeout(timeoutId);
      }
    });
  }

  loadRepositories(showLoading: boolean = true): void {
    if (showLoading) {
      this.loading = true;
    }
    this.errorRepositories = {};
    const timeoutId = setTimeout(() => {
      if (showLoading) {
        this.loading = false;
      }
      this.repositories.forEach(repository => {
        if (!(repository.uuid in this.errorRepositories)) {
          this.errorRepositories[repository.uuid] = true;
        }
      });
    }, 5000);

    this.http.get<any>(`${this.repositoriesUrl}?page=1&itemsPerPage=10`).subscribe(
      data => {
        this.repositories = data['hydra:member'];
        let remainingRepositories = this.repositories.length;

        this.repositories.forEach(repository => {
          this.loadRepositoryStatus(repository.uuid, (errorOccurred: boolean) => {
            remainingRepositories--;

            this.errorRepositories[repository.uuid] = errorOccurred;

            if (remainingRepositories === 0) {
              if (showLoading) {
                this.loading = false;
              }
              clearTimeout(timeoutId);
            }
          });
        });
      },
      error => {
        this.loading = false;
        this.repositories.forEach(repository => {
          this.errorRepositories[repository.uuid] = true;
        });
        clearTimeout(timeoutId);
      }
    );
  }

  loadRepositoryStatus(repositoryUuid: string, callback: (errorOccurred: boolean) => void): void {
    const timeoutId = setTimeout(() => {
      callback(true);
    }, 5000);
    this.http.get<any>(`${this.baseUrl}/image-repositories/server/${repositoryUuid}/status`).subscribe(
      data => {
        const output = data.output;
        this.repositoryStatuses[repositoryUuid] = {
          ...output,
          disk: {
            ...output.disk,
            used: parseFloat(output.disk.used),
            available: parseFloat(output.disk.available)
          },
          ram: {
            ...output.ram,
            used: parseFloat(output.ram.used),
            available: parseFloat(output.ram.available)
          }
        };
        clearTimeout(timeoutId);
        callback(false);
      },
      error => {
        this.toastService.error(error.error['hydra:description'] || 'Error al cargar el estado del repositorio');
        clearTimeout(timeoutId);
        callback(true);
      }
    );
  }

  loadOgBootStatus(): void {
    this.isDhcp = false;
    this.loadStatus(this.ogBootApiUrl, this.ogBootDiskUsage, this.ogBootServicesStatus, this.ogBootDiskUsageChartData, this.installedOgLives, this.isDhcp, 'errorOgBoot', false);
  }

  loadDhcpStatus(): void {
    this.isDhcp = true;
    this.loadStatus(this.dhcpApiUrl, this.dhcpDiskUsage, this.dhcpServicesStatus, this.dhcpDiskUsageChartData, this.installedOgLives, this.isDhcp, 'errorDhcp', false);
  }

  onTabChange(event: MatTabChangeEvent): void {
    switch (event.index) {
      case 0:
        this.loadOgBootStatus();
        break;
      case 1:
        this.loadDhcpStatus();
        break;
      case 2:
        if (this.repositories.length === 0) {
          this.loadRepositories(false);
        }
        break;
      default:
        break;
    }
  }

  onRepositoryChange(repositoryUuid: string): void {
    this.selectedRepositoryUuid = repositoryUuid;
    if (repositoryUuid && !this.repositoryStatuses[repositoryUuid]) {
      this.loadRepositoryStatus(repositoryUuid, (errorOccurred: boolean) => {
        if (errorOccurred) {
          this.errorRepositories[repositoryUuid] = true;
        }
      });
    }
  }

  getSelectedRepositoryName(): string {
    const selectedRepo = this.repositories.find(repo => repo.uuid === this.selectedRepositoryUuid);
    return selectedRepo ? selectedRepo.name : '';
  }

  refreshAll(): void {
    this.loading = true;
    this.updateLastUpdateTime();
    this.loadStatus(this.ogBootApiUrl, this.ogBootDiskUsage, this.ogBootServicesStatus, this.ogBootDiskUsageChartData, this.installedOgLives, false, 'errorOgBoot', true);
    this.loadStatus(this.dhcpApiUrl, this.dhcpDiskUsage, this.dhcpServicesStatus, this.dhcpDiskUsageChartData, this.installedOgLives, true, 'errorDhcp', true);
    this.loadRepositories(true);
    this.syncSubnets();
    this.syncTemplates();
    this.syncOgLives();
    this.toastService.success('Datos actualizados correctamente');
  }

  updateLastUpdateTime(): void {
    const now = new Date();
    this.lastUpdateTime = now.toLocaleTimeString('es-ES', {
      hour: '2-digit',
      minute: '2-digit',
      second: '2-digit'
    });
  }

  getCurrentTime(): string {
    const now = new Date();
    return now.toLocaleTimeString('es-ES', {
      hour: '2-digit',
      minute: '2-digit',
      second: '2-digit'
    });
  }

  retryRepositoryStatus(repositoryUuid: string): void {
    this.loadRepositoryStatus(repositoryUuid, (errorOccurred: boolean) => {
      this.errorRepositories[repositoryUuid] = errorOccurred;
      if (!errorOccurred) {
        this.toastService.success('Estado del repositorio actualizado correctamente');
      } else {
        this.toastService.error('Error al cargar el estado del repositorio');
      }
    });
  }
}
