import { Component, Inject, OnInit } from '@angular/core';
import { MAT_DIALOG_DATA } from '@angular/material/dialog';

interface Partition {
  diskNumber: number;
  partitionNumber: number;
  partitionCode: string;
  size: number;
  filesystem: string | null;
  isSummary?: boolean;
}

interface SimplifiedClient {
  name: string;
  partitions: Partition[];
}

interface GroupedClientPartitions {
  clientNames: string[];
  partitions: Partition[];
}

@Component({
  selector: 'app-partition-type-organizator',
  templateUrl: './partition-type-organizator.component.html',
  styleUrl: './partition-type-organizator.component.css'
})
export class PartitionTypeOrganizatorComponent implements OnInit {
  displayedColumns: string[] = ['diskNumber', 'partitionNumber', 'partitionCode', 'size', 'filesystem'];
  groupedPartitions: GroupedClientPartitions[] = [];

  constructor(@Inject(MAT_DIALOG_DATA) public data: SimplifiedClient[]) { }

  ngOnInit(): void {
    const simplifiedClients = this.simplifyClients(this.data);
    this.groupedPartitions = this.groupClientsByPartitions(simplifiedClients);
  }

  private simplifyClients(clients: SimplifiedClient[]): SimplifiedClient[] {
    return clients.map(client => {
      const partitionZero = client.partitions.find(p => p.partitionNumber === 0);
      const otherPartitions = client.partitions.filter(p => p.partitionNumber !== 0);

      const simplifiedPartitions: Partition[] = otherPartitions.map(p => ({
        diskNumber: p.diskNumber,
        partitionNumber: p.partitionNumber,
        partitionCode: p.partitionCode,
        size: p.size,
        filesystem: p.filesystem,
        isSummary: false
      }));

      if (partitionZero) {
        simplifiedPartitions.push({
          diskNumber: partitionZero.diskNumber,
          partitionNumber: partitionZero.partitionNumber,
          partitionCode: partitionZero.partitionCode,
          size: partitionZero.size,
          filesystem: null,
          isSummary: true
        });
      }

      return {
        name: client.name,
        partitions: simplifiedPartitions
      };
    });
  }

  private groupClientsByPartitions(clients: SimplifiedClient[]): GroupedClientPartitions[] {
    const groups: GroupedClientPartitions[] = [];

    clients.forEach(client => {
      const normalizedPartitions = this.normalizePartitions(client.partitions);

      const existingGroup = groups.find(group =>
        JSON.stringify(this.normalizePartitions(group.partitions)) === JSON.stringify(normalizedPartitions)
      );

      if (existingGroup) {
        existingGroup.clientNames.push(client.name);
      } else {
        groups.push({
          clientNames: [client.name],
          partitions: client.partitions
        });
      }
    });

    return groups;
  }

  private normalizePartitions(partitions: Partition[]): Partition[] {
    return [...partitions].sort((a, b) => a.partitionNumber - b.partitionNumber);
  }
}
